NSIS_FILE := nsis-2.44.zip
NSIS_URL := http://easynews.dl.sourceforge.net/sourceforge/nsis/$(NSIS_FILE)
NSIS_BASE_DIR := $(BUILD_BASE)/nsis
NSIS_DIR := $(NSIS_BASE_DIR)/nsis-2.44
NSIS_TARGET := ../livecpd2-installer-win32.exe
NSIS_SOURCE := $(LIVECPD2)/boot/livecpd2.nsi
NSIS_VERSION := $(shell date -u '+%Y-%m-%dT%H:%M:%SZ')

NSIS_SYSLINUX_FILE := syslinux-4.04.zip
NSIS_SYSLINUX_URL := http://www.kernel.org/pub/linux/utils/boot/syslinux/$(NSIS_SYSLINUX_FILE)
NSIS_MAKEBOOTFAT_FILE := makebootfat-1.4-windows-pentium.zip
NSIS_MAKEBOOTFAT_URL := http://easynews.dl.sourceforge.net/sourceforge/advancemame/$(NSIS_MAKEBOOTFAT_FILE)

NSIS_SOURCE_TARGET := $(shell perl -e 'foreach (split /\s+/,"$(NSIS_SOURCE)") { s!^.*/([^/]+)$$!$(NSIS_BASE_DIR)/$$1 !; print; }' )

NSIS_SYSROOT_FILES := \
		$(LIVECPD2)/boot/syslinux.cfg \
		$(LIVECPD2)/boot/bootmsg.txt \
		$(LIVECPD2)/initrd/initrd.gz \
		$(LIVECPD2)/kernel/bzImage \
		$(LIVECPD2)/root/root.img

NSIS_BASELINE_FILES := \
		$(LIVECPD2)/toolchain/boot/ldlinux.bss \
		$(LIVECPD2)/toolchain/boot/ldlinux.sys \
		$(LIVECPD2)/toolchain/boot/mbrfat.bin \
		$(NSIS_SYSROOT_FILES)

NSIS_SIZE := $(shell du -cm $(NSIS_BASELINE_FILES) | grep "total" | cut -f 1)

.PHONY: nsis nsis-clean nsis-spotless nsis-distclean

nsis: $(NSIS_TARGET)

$(NSIS_TARGET): .package.nsis.toolcheck .package.nsis.ex $(NSIS_SOURCE) $(NSIS_BASELINE_FILES)
	-rm -f $@
	-rm -f livecpd2.exe
	mkdir -p $(NSIS_BASE_DIR)
	echo $(NSIS_SOURCE_TARGET) $(NSIS_SOURCE)
	set -e; for file in $(NSIS_BASELINE_FILES) $(NSIS_SOURCE); do \
		target=`echo $$file | perl -pe 's!^.*/([^/]+)$$!$$1!'`; \
		rm -f $(NSIS_BASE_DIR)/$$target || true; \
		ln -s $$file $(NSIS_BASE_DIR)/$$target; \
	done
	touch $(NSIS_BASE_DIR)/override.img
	( cd $(NSIS_DIR); wine makensis.exe \
	  		"/DSIZE=$(NSIS_SIZE)" \
			"/DVERSION=$(NSIS_VERSION)" -- \
			$(NSIS_SOURCE_TARGET) )
	cp $(NSIS_BASE_DIR)/livecpd2.exe $@
	-rm -f $(NSIS_BASE_DIR)/livecpd2.exe
	chmod 0775 $@
	touch $@


nsis-clean:
	-rm -f $(NSIS_TARGET)

nsis-spotless: nsis-clean
	-rm -f .package.nsis.ex
	-rm -rf $(NSIS_BASE_DIR)

nsis-distclean: nsis-spotless
	-rm -f .package.nsis.dl
	-rm -f $(DLLOC)/$(NSIS_FILE) $(DLLOC)/$(NSIS_SYSLINUX_FILE) $(DLLOC)/$(NSIS_MAKEBOOTFAT_FILE)

.package.nsis.dl:
	-rm -f $@
	-mkdir -p $(DLLOC)
	wget -O $(DLLOC)/$(NSIS_FILE) $(NSIS_URL)
	wget -O $(DLLOC)/$(NSIS_SYSLINUX_FILE) $(NSIS_SYSLINUX_URL)
	wget -O $(DLLOC)/$(NSIS_MAKEBOOTFAT_FILE) $(NSIS_MAKEBOOTFAT_URL)
	touch $@

.package.nsis.ex: .package.nsis.dl
	-rm -f $@
	-rm -rf $(NSIS_DIR)
	mkdir -p $(NSIS_BASE_DIR)
	( cd $(NSIS_BASE_DIR); unzip $(DLLOC)/$(NSIS_FILE) )
	( cd $(NSIS_BASE_DIR); unzip -j $(DLLOC)/$(NSIS_SYSLINUX_FILE) win32/syslinux.exe )
	( cd $(NSIS_BASE_DIR); unzip -j $(DLLOC)/$(NSIS_MAKEBOOTFAT_FILE) makebootfat.exe )
	touch $@

.package.nsis.toolcheck:
	-@rm -f $@
	@if ! which wine >/dev/null 2>&1; then \
		echo "You need WINE installed and configured to build the NSIS installer"; \
		echo "  Ensure $(NSIS_BASE_DIR) or super directory is available to WINE" \
		exit 1; \
	fi
	@touch $@
