#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mount.h>
#include <dirent.h>
#include <errno.h>

#include "filesys.h"

#define PROBE_DEVICE_PATH	"/tmp/probe_device"
#define PROBE_MOUNT_PATH	"/base"

char *g_probe_root_paths[] = { 
	"/root.img", 
	"/isolinux/root.img" 
};
char *g_probe_override_paths[] = { 
	"/override.img", 
	"/linux/override.img",
	"/isolinux/override.img",
	"/aer/override.img" 
};

extern int g_root_setup;

extern char *g_root_path;
extern int g_root_major;
extern int g_root_minor;

extern char *g_override_path;
extern int g_override_major;
extern int g_override_minor;

static void add_root_path( int major, int minor, char *path ) {
	printf("Added root at %d:%d%s\n", major, minor, path);
	g_root_path = path;
	g_root_major = major;
	g_root_minor = minor;
}

static void add_override_path( int major, int minor, char *path ) {
	printf("Added override at %d:%d%s\n", major, minor, path);
	g_override_path = path;
	g_override_major = major;
	g_override_minor = minor;
}

static int probe_device( int major, int minor ) {
	int rc;
	int i;
	struct stat st;
	char cf[256];
	
	if (!auto_fs_mount(PROBE_DEVICE_PATH, PROBE_MOUNT_PATH, MS_NOATIME|MS_RDONLY|MS_NODIRATIME))
		return 0;

	rc = 0;

    if (!g_root_setup) {
        for (i=0; i<(sizeof(g_probe_root_paths)/sizeof(g_probe_root_paths[0])); i++) {
            snprintf(cf, 256, "%s/%s", PROBE_MOUNT_PATH, g_probe_root_paths[i]);
            if (stat(cf, &st) != 0) continue;
            if (st.st_size == 0) continue;

            rc = 1;
            add_root_path(major, minor, g_probe_root_paths[i]);
        }
    }

	for (i=0; i<(sizeof(g_probe_override_paths)/sizeof(g_probe_override_paths[0])); i++) {
		snprintf(cf, 256, "%s/%s", PROBE_MOUNT_PATH, g_probe_override_paths[i]);
		if (stat(cf, &st) != 0) continue;
		/* if (st.st_size == 0) continue; */

		rc = 1;
		add_override_path(major, minor, g_probe_override_paths[i]);
	}

	if (umount(PROBE_MOUNT_PATH) != 0) {
		fprintf(stderr, "Unable to unmount probe device: %s\n", strerror(errno));
		exit(1);
	}
	return rc;
}

int probe_sys_path( char *path ) {
	char *probe_dev_path;
	int fd, n;
	char line[64];
	char *q;
	int major, minor;
	struct stat st;
	dev_t dt;

	/* Quick existance check */	
	if (stat(path, &st) != 0) return 0;

	/* Open device file and read "major:minor" */
	probe_dev_path = (char *)malloc(strlen(path) + 5);
	sprintf(probe_dev_path, "%s/dev", path);	
	fd=open(probe_dev_path, O_RDONLY);
	free(probe_dev_path);
	if (fd == -1) return 0;
	n = read(fd, line, sizeof(line)-1);
	close(fd);
	if (n == 0) return 0;
	line[n] = '\0';

	/* Get major and minor */
	if ((q=strpbrk(line, ":")) == NULL) return 0;
	*q = '\0';
	q++;
	if (!line[0] || !q[0]) return 0;
	major = atoi(line);
	minor = atoi(q);

    /* Ignore loopback devices */
    if (major == 7) return 0;

	/* Setup a device file */
	unlink(PROBE_DEVICE_PATH);
	dt = ((major & 0xFF) << 8) | (minor & 0xFF);	/* FIXME */
	if (mknod(PROBE_DEVICE_PATH, S_IFBLK, dt) != 0) return 0;
	if (chmod(PROBE_DEVICE_PATH, 0666) != 0) return 0;

	printf("Probing device %d:%d...\n", major, minor);

	return probe_device(major, minor);
}

static void probe_subdirs( char *base ) {
	DIR *d = opendir(base);
	struct dirent *de;
	struct stat s;
	char *fn = NULL;
	
	if (!d) return;

	while ((de=readdir(d)) != NULL) {
		if (!strcmp(de->d_name, ".") || !strcmp(de->d_name, "..")) continue;
		if (fn != NULL) free(fn);
		fn = (char *)malloc(strlen(base) + strlen(de->d_name) + 2);
		sprintf(fn, "%s/%s", base, de->d_name);

		if (lstat(fn, &s) != 0) continue;
		if (!S_ISDIR(s.st_mode)) continue;

		probe_sys_path(fn);
		probe_subdirs(fn);
	}

	if (fn != NULL) free(fn);
	closedir(d);
}

void probe_all_devices( void ) {
	probe_subdirs("/sys/block");
}
